/*
    These two functions are a straightforward implementation
    of the Merge and Mergesort algorithms from pages 31
    and 34 of our textbook.
*/
#include <limits.h>  // needed for INT_MAX (the sentinel)


void merge(int* inputArray, int p, int q, int r)
{
   // pointers to the two temporary arrays needed for merging
   int* leftArray;
   int* rightArray;
   // dimensions for the two temporary arrays
   int  leftSize = q - p + 2;  // an extra space for the sentinel
   int rightSize = r - q + 1;  // an extra space for the sentinel

   int leftIndex, rightIndex;
   int i;

   // allocate the two temporary arrays
   leftArray  = (int*)malloc(leftSize*sizeof(int));
   if (leftArray == NULL)  // error check
   {
      fprintf(stderr, "merge: not enough memory for temporary array.\n");
      exit(-1);
   }
   rightArray = (int*)malloc(rightSize*sizeof(int));
   if (rightArray == NULL)  // error check
   {
      fprintf(stderr, "merge: not enough memory for temporary array.\n");
      free(leftArray);
      exit(-1);
   }

   // copy the two parts of the input array into the two temporary arrays
   for (i = 0; i < leftSize-1; i++)
   {
      leftArray[i]  = inputArray[p+i];
   }
   for (i = 0; i < rightSize-1; i++)
   {
      rightArray[i] = inputArray[q+1+i];
   }

   // put the sentinel value at the end of each temporary array
   leftArray[leftSize-1]   = INT_MAX;
   rightArray[rightSize-1] = INT_MAX;

   // merge values from the two temporary arrays back into the input array
   leftIndex = 0;
   rightIndex = 0;
   for (i = p; i <= r; i++)
   {
      if (leftArray[leftIndex] <= rightArray[rightIndex])
      {
         inputArray[i] = leftArray[leftIndex];
         leftIndex++;
      }
      else
      {
         inputArray[i] = rightArray[rightIndex];
         rightIndex++;
      }
   }

   // deallocate the temporary arrays
   free(leftArray);
   free(rightArray);
}//merge()



void mergeSort(int* inputArray, int p, int r)
{
   int q; //index separating the left and right parts of the input array

   // check for a trivial array
   if ( r-p < 1 )
   {
      return;
   }
   else
   {
      q = (p+r-1)/2; // when the input array has odd length, the right part is larger

      // recursively sort the subarrays
      mergeSort(inputArray, p, q);   // sort the left part
      mergeSort(inputArray, q+1, r); // sort the right part

      // merge the two sorted parts together
      merge(inputArray, p, q, r);
   }
}//mergeSort()
